package NDash::Groups;

use Moose;
use FindBin qw($Bin);
use File::Spec;
use File::Basename;
use File::Slurp qw(slurp);
use Carp;

has 'file'          => ( is => 'ro', isa => 'Str', default => sub {
  for ('/etc/ndash/groups', File::Spec->catfile(dirname($Bin), 'conf', 'groups')) {
    return $_ if -f $_;
  }
  croak "Cannot find groups file - aborting";
});
has 'groups'        => ( is => 'ro', isa => 'ArrayRef', auto_deref => 1, lazy_build => 1 ); 
has 'group_hosts'   => ( is => 'rw', isa => 'HashRef' );

sub _build_groups {
  my $self = shift;

  my @groups = ();
  my %group_hosts = ();
  for my $entry (grep(/\S/, grep(m/^[^#]/, slurp($self->file)))) {
    chomp $entry;

    # Split $entry and sanity-check
    my ($group, $hosts, $exec) = split /\s*:\s*/, $entry, 3;
    die "Bogus groups entry '$entry'\n" unless $group;
    unless ($hosts || $exec) {
      warn "Group '$group' has no hosts defined\n";
      next;
    }

    # Get lists of hosts for this group, either static, or via executable
    my @hosts;
    push @hosts, split /\s*,\s*/, $hosts if $hosts;
    my $exec_hosts = qx($exec) if $exec;
    push @hosts, split /\s*[\s,]\s*/, $exec_hosts if $exec_hosts;

    # Collate
    push @groups, $group;
    $group_hosts{ $group } = \@hosts;
  }
  $self->group_hosts( \%group_hosts );

  return \@groups;
}

sub hosts
{
  my ($self, $group) = @_;
  croak "Missing 'group' argument'" unless $group;
  croak "Invalid group '$group'" unless $self->group_hosts->{$group};
  return wantarray ? @{ $self->group_hosts->{$group} } : $self->group_hosts->{$group};
}

1;

=head1 NAME

NDash::Groups - class modelling NDash groups file

=head1 SYNOPSIS

    use NDash::Groups;

    $ng = NDash::Groups->new;

    @groups = $ng->groups;
    @hosts = $ng->hosts( $group[0] );

=head1 AUTHOR

Gavin Carr E<lt>gavin@openfusion.com.auE<gt>

=head1 COPYRIGHT AND LICENCE

Copyright 2009 Gavin Carr.

This software is free software, licensed under the same terms as Perl itself.

=cut

